<?php

namespace Drupal\forms_steps\Controller;

use Drupal\Core\Controller\ControllerBase;

/**
 * Class FormsStepsController.
 *
 * @package Drupal\forms_steps\Controller
 */
class FormsStepsController extends ControllerBase {

  /**
   * Display the step form.
   *
   * @param int $forms_steps
   *   Forms Steps id to display step from.
   * @param mixed $step
   *   Step to display.
   * @param null|int $nid
   *   Node id to load if one exists.
   *
   * @return mixed
   *   Form that match the input parameters.
   */
  public function step($forms_steps, $step, $nid = NULL) {
    return self::getForm($forms_steps, $step, $nid);
  }

  /**
   * Get a form based on the $step and $nid. If $nid is empty or not existing
   * we provide a create form. edit otherwise.
   *
   * TODO: De we need to move it in a service?
   *
   * @param int $forms_steps
   *   Forms Steps id to get the form from.
   * @param mixed $step
   *   Step to get the Form from.
   * @param null|int $nid
   *   Node id to load if one exists.
   *
   * @return mixed
   *   Form that match the input parameters.
   */
  public static function getForm($forms_steps, $step, $nid = NULL) {
    /** @var \Drupal\forms_steps\Entity\FormsSteps $formsSteps */
    $formsSteps = \Drupal::entityTypeManager()
      ->getStorage('forms_steps')
      ->load($forms_steps);

    if (!$formsSteps->hasStep($step)) {
      // TODO: Propose a better error management.
      throw new \InvalidArgumentException("The Step '$step' does not exist in forms steps '{$forms_steps}'");
    }

    $step = $formsSteps->getStep($step);

    $entity_type = 'node';
    $entity_key_type = \Drupal::entityTypeManager()
      ->getDefinition($entity_type)
      ->getKey('bundle');

    // We create the entity
    $entity = NULL;
    if (!is_null($nid)) {
      $entity = \Drupal::entityTypeManager()
        ->getStorage($entity_type)
        ->load($nid);
    }

    if (is_null($entity)) {
      $entity = \Drupal::entityTypeManager()
        ->getStorage($entity_type)
        ->create([$entity_key_type => $step->nodeType()]);
    }

    $form = \Drupal::service('entity.form_builder')
      ->getForm($entity, preg_replace("/^$entity_type\./", '', $step->formMode()));


    if ($step->hideDelete()) {
      unset($form['actions']['delete']);
    }
    elseif ($step->deleteLabel()) {
      $form['actions']['delete']['#title'] = t($step->deleteLabel());
    }

    return $form;
  }

}
