<?php

namespace Drupal\forms_steps\Form;

use Drupal\Core\Form\FormState;

/**
 * Class FormsStepsAlter.
 *
 * @package Drupal\forms_steps\Form
 */
class FormsStepsAlter {

  /**
   * Redirect to the next step if one exists.
   *
   * @param array $form
   *   Form to alter.
   * @param \Drupal\Core\Form\FormState $form_state
   *   Form State to update.
   */
  public static function setNextRoute(array &$form, FormState $form_state) {
    /** @var \Drupal\forms_steps\Service\FormsStepsManager $formsStepsManager */
    $formsStepsManager = \Drupal::service('forms_steps.manager');
    $nextRoute = $formsStepsManager->getNextStepRoute(\Drupal::routeMatch()
      ->getRouteName());

    if ($nextRoute) {
      $form_state->setRedirect($nextRoute, [
        'nid' => $form_state->getFormObject()
          ->getEntity()
          ->id(),
      ]);
    }
  }

  /**
   * Define the submit and cancel label using the step configuration.
   *
   * @param array $form
   *   Form to alter.
   * @param \Drupal\Core\Form\FormState $form_state
   *   Forms State to Update.
   */
  public static function setButtonLabel(array &$form, FormState $form_state) {
    /** @var \Drupal\forms_steps\Service\FormsStepsManager $formsStepsManager */
    $formsStepsManager = \Drupal::service('forms_steps.manager');

    /** @var \Drupal\forms_steps\Step $step */
    $step = $formsStepsManager->getStepByRoute(\Drupal::routeMatch()
      ->getRouteName());

    if ($step) {
      if ($step->submitLabel()) {
        $form['actions']['submit']['#value'] = t($step->submitLabel());
      }

      if ($step->cancelLabel()) {
        $form['actions']['cancel']['#value'] = t($step->cancelLabel());
      }
    }
  }

}
